//+------------------------------------------------------------------+
//|                                                     myRandom.mq5 |
//|                                   Copyright  2009 EarnForex.com |
//|                                        http://www.earnforex.com/ |
//+------------------------------------------------------------------+
#property copyright "Copyright  2009, EarnForex.com"
#property link      "http://www.earnforex.com"
#property description "myRandom - places position in random directions. Adds up to a set volume."
#property description "You can control stop-loss, take-profit, volume and entry period."

#include <Trade/Trade.mqh>

input double Lots = 1; //Position will be incremented with this volume
input int Slippage = 3;
input int RandomEntryPeriod = 1; //In bars. How many bars to wait before adding or cutting from the position.
input int StopLoss = 90;
input int TakeProfit = 90;
input double MaxVolume = 4.0; //Maximum volume of the position.

int Magic;
int LastBars = 0;
int OrderTaken = 0;

double Poin;
int Deviation;

//Main position class
CTrade *Trade;

void OnInit()
{
   //Checking for unconvetional Point digits number
   if ((_Point == 0.00001) || (_Point == 0.001))
   {
      Poin = _Point * 10;
      Deviation = Slippage * 10;
   }
   else
   {
      Poin = _Point; //Normal
      Deviation = Slippage;
   }
   Magic = Period() + 1339005;

	Trade = new CTrade;
	Trade.SetDeviationInPoints(Deviation);
}

void OnDeinit(const int reason)
{
	delete Trade;
}

//+------------------------------------------------------------------+
//| Random entry expert advisor                                      |
//+------------------------------------------------------------------+
void OnTick()
{
	double Vol = 0;
	ENUM_POSITION_TYPE Dir;
	if (LastBars == Bars(_Symbol, _Period)) return;
	else LastBars = Bars(_Symbol, _Period);
   if (AccountInfoDouble(ACCOUNT_FREEMARGIN) < (Lots*2*1000))
   {
   	Print("Account Free Margin Insufficient. Should have at least Lots * 2 * 1000 currency units.");
   	return;
   }

   if (Lots < 0.1)
   {
   	Print("Lots < 0.1. Exiting.");
   	return;
  	}

   if (PositionSelect(_Symbol)) //We have a position
   {
      Vol = PositionGetDouble(POSITION_VOLUME);
      Dir = PositionGetInteger(POSITION_TYPE);
   }
   
   MathSrand(TimeLocal());
   
   if (Bars(_Symbol, _Period) >= OrderTaken + RandomEntryPeriod)
   {
	  if ((MathRand()%2) == 1)
	  {
		  if (!((Dir == POSITION_TYPE_SELL) && (Vol >= MaxVolume))) fSell();
	  }
	  else
	  {
		  if (!((Dir == POSITION_TYPE_BUY) && (Vol >= MaxVolume))) fBuy();
	  }
   }
}

//+------------------------------------------------------------------+
//| Buy                                                              |
//+------------------------------------------------------------------+
void fBuy()
{
	double Ask = SymbolInfoDouble(Symbol(), SYMBOL_ASK);
	Trade.PositionOpen(_Symbol, ORDER_TYPE_BUY, Lots, Ask, Ask - StopLoss * Poin, Ask + TakeProfit * Poin, "myRandom");
}

//+------------------------------------------------------------------+
//| Sell                                                             |
//+------------------------------------------------------------------+
void fSell()
{
	double Bid = SymbolInfoDouble(Symbol(), SYMBOL_BID);
	Trade.PositionOpen(_Symbol, ORDER_TYPE_SELL, Lots, Bid, Bid + StopLoss * Poin, Bid - TakeProfit * Poin, "myRandom");
}

//Deletes two pending orders
void RemovePendings(string dir)
{
   long last_type; //Last type of order deleted
   int count = 0; //Total amount of orders deleted
   int total = OrdersTotal();
   for (int i = total-1; i >= 0; i--)
   {
      ulong ticket = OrderGetTicket(i);
      long otype = OrderGetInteger(ORDER_TYPE);
      if ((
         //Take-profit and Stop-losses for previous SELL position volume
         (((otype == ORDER_TYPE_BUY_LIMIT) || (otype == ORDER_TYPE_BUY_STOP)) && (dir == "Buy"))
         ||
         //Take-profit and Stop-losses for previous BUY position volume
         (((otype == ORDER_TYPE_SELL_LIMIT) || (otype == ORDER_TYPE_SELL_STOP)) && (dir == "Sell"))
         ) && (OrderGetInteger(ORDER_MAGIC) == Magic) && (otype != last_type))
      {
         MqlTradeRequest Request;
	      Request.action = TRADE_ACTION_REMOVE;
         Request.order = ticket;
 
         MqlTradeResult Result;
      
       	bool result = OrderSend(Request, Result); 
	      if (result == false)	Print("Incorrect Request/Result Structure When Removing Pending ", ticket);
	      else
	      {
	         Print("Result Code: ", Result.retcode, ", ", "Comment: ", Result.comment);
	         last_type = otype;
	         count++;
	      }
      }
      if (count == 2) return;
   }
}
